package com.gic.enterprise.context;

import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.utils.UserDetail;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.concurrent.TimeUnit;

public class UserContext {
    /**
     * 有效期4小时
     */
    public static final Long expireLength = 4L;
    private static UserContext userContext = null;
    private static final Logger log = LogManager.getLogger(UserContext.class);

    public static UserContext getContext(){
        if (null == userContext) {
            synchronized (UserContext.class) {
                if (null == userContext) {
                    try {
                        userContext = new UserContext();
                    } catch (Exception e) {
                        log.info("创建对象失败");
                    }
                }
            }
        }
        return userContext;
    }

    public UserDetail get(){
        return (UserDetail) RequestContext.getContext().getRequest().getAttribute("user");
    }

    public UserDetail getByRedis(){
        String token = getToken();
        return getByToken(token);
    }

    public UserDetail getByToken(String token) {
        if(StringUtils.isBlank(token)){
            return new UserDetail();
        }
        String key = Constants.TOKEN_KEY + token;
        Object cache = RedisUtil.getCache(key);
        if(cache != null){
            return (UserDetail) cache;
        }
        return new UserDetail();
    }

    /**
     * 返回token值
     * @Title: init 
     * @Description:
      * @author guojuxing
     * @param userDetail 
     * @return  java.lang.String 
      */
    public String init(UserDetail userDetail){
        String uuId = ToolUtil.randomUUID();
        String key = Constants.TOKEN_KEY+uuId;
        RedisUtil.setCache(key, userDetail, expireLength, TimeUnit.HOURS);
        Cookie cookie = new Cookie(Constants.USER_TOKEN, uuId);
        cookie.setPath("/");
        HttpServletResponse response = RequestContext.getContext().getResponse();
        if(response != null){
            response.addCookie(cookie);
        }
        return uuId;
    }

    public void refresh(){
        String key = Constants.TOKEN_KEY + getToken();
        RedisUtil.getRedisClient().getBucket(key).expire(expireLength, TimeUnit.HOURS);
    }

    public void destory(){
        String key = Constants.TOKEN_KEY + getToken();
        RedisUtil.delCache(key);
    }

    public String getToken(){
        String token = null;
        HttpServletRequest request = RequestContext.getContext().getRequest();
        if (request != null){
            Cookie[] cookies = request.getCookies();
            if(cookies != null){
                for (Cookie cookie : cookies) {
                    if (Constants.USER_TOKEN.equals(cookie.getName())) {
                        token = cookie.getValue();
                        break;
                    }
                }
            }
        }
        return token;
    }
}
